'use strict';

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var path = require('path');
var fs = require('./utils/fs');
var objectHash = require('./utils/objectHash');
var md5 = require('./utils/md5');
var isURL = require('./utils/is-url');
var sanitizeFilename = require('sanitize-filename');
var config = require('./utils/config');

var ASSET_ID = 1;

/**
 * An Asset represents a file in the dependency tree. Assets can have multiple
 * parents that depend on it, and can be added to multiple output bundles.
 * The base Asset class doesn't do much by itself, but sets up an interface
 * for subclasses to implement.
 */

var Asset = function () {
  function Asset(name, pkg, options) {
    (0, _classCallCheck3.default)(this, Asset);

    this.id = ASSET_ID++;
    this.name = name;
    this.basename = path.basename(this.name);
    this.package = pkg || {};
    this.options = options;
    this.encoding = 'utf8';
    this.type = path.extname(this.name).slice(1);

    this.processed = false;
    this.contents = options ? options.contents : null;
    this.ast = null;
    this.generated = null;
    this.hash = null;
    this.parentDeps = new _set2.default();
    this.dependencies = new _map2.default();
    this.depAssets = new _map2.default();
    this.parentBundle = null;
    this.bundles = new _set2.default();
    this.cacheData = {};
  }

  (0, _createClass3.default)(Asset, [{
    key: 'shouldInvalidate',
    value: function shouldInvalidate() {
      return false;
    }
  }, {
    key: 'loadIfNeeded',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!(this.contents == null)) {
                  _context.next = 4;
                  break;
                }

                _context.next = 3;
                return this.load();

              case 3:
                this.contents = _context.sent;

              case 4:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function loadIfNeeded() {
        return _ref.apply(this, arguments);
      }

      return loadIfNeeded;
    }()
  }, {
    key: 'parseIfNeeded',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return this.loadIfNeeded();

              case 2:
                if (this.ast) {
                  _context2.next = 6;
                  break;
                }

                _context2.next = 5;
                return this.parse(this.contents);

              case 5:
                this.ast = _context2.sent;

              case 6:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function parseIfNeeded() {
        return _ref2.apply(this, arguments);
      }

      return parseIfNeeded;
    }()
  }, {
    key: 'getDependencies',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return this.loadIfNeeded();

              case 2:
                if (!(this.contents && this.mightHaveDependencies())) {
                  _context3.next = 6;
                  break;
                }

                _context3.next = 5;
                return this.parseIfNeeded();

              case 5:
                this.collectDependencies();

              case 6:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function getDependencies() {
        return _ref3.apply(this, arguments);
      }

      return getDependencies;
    }()
  }, {
    key: 'addDependency',
    value: function addDependency(name, opts) {
      this.dependencies.set(name, (0, _assign2.default)(this.dependencies.get(name) || {}, { name: name }, opts));
    }
  }, {
    key: 'addURLDependency',
    value: function addURLDependency(url) {
      var from = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.name;
      var opts = arguments[2];

      if (!url || isURL(url)) {
        return url;
      }

      if ((typeof from === 'undefined' ? 'undefined' : (0, _typeof3.default)(from)) === 'object') {
        opts = from;
        from = this.name;
      }

      var resolved = path.resolve(path.dirname(from), url).replace(/[?#].*$/, '');
      this.addDependency('./' + path.relative(path.dirname(this.name), resolved), (0, _assign2.default)({ dynamic: true }, opts));

      return this.options.parser.getAsset(resolved, this.package, this.options).generateBundleName();
    }
  }, {
    key: 'getConfig',
    value: function () {
      var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(filenames) {
        var conf;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return config.resolve(this.name, filenames);

              case 2:
                conf = _context4.sent;

                if (!conf) {
                  _context4.next = 8;
                  break;
                }

                // Add as a dependency so it is added to the watcher and invalidates
                // this asset when the config changes.
                this.addDependency(conf, { includedInParent: true });
                _context4.next = 7;
                return config.load(this.name, filenames);

              case 7:
                return _context4.abrupt('return', _context4.sent);

              case 8:
                return _context4.abrupt('return', null);

              case 9:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function getConfig(_x2) {
        return _ref4.apply(this, arguments);
      }

      return getConfig;
    }()
  }, {
    key: 'mightHaveDependencies',
    value: function mightHaveDependencies() {
      return true;
    }
  }, {
    key: 'load',
    value: function () {
      var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5() {
        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _context5.next = 2;
                return fs.readFile(this.name, this.encoding);

              case 2:
                return _context5.abrupt('return', _context5.sent);

              case 3:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function load() {
        return _ref5.apply(this, arguments);
      }

      return load;
    }()
  }, {
    key: 'parse',
    value: function parse() {
      // do nothing by default
    }
  }, {
    key: 'collectDependencies',
    value: function collectDependencies() {
      // do nothing by default
    }
  }, {
    key: 'pretransform',
    value: function () {
      var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6() {
        return _regenerator2.default.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function pretransform() {
        return _ref6.apply(this, arguments);
      }

      return pretransform;
    }()
  }, {
    key: 'transform',
    value: function () {
      var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7() {
        return _regenerator2.default.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      function transform() {
        return _ref7.apply(this, arguments);
      }

      return transform;
    }()
  }, {
    key: 'generate',
    value: function generate() {
      return (0, _defineProperty3.default)({}, this.type, this.contents);
    }
  }, {
    key: 'install',
    value: function () {
      var _ref9 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8() {
        return _regenerator2.default.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                return _context8.abrupt('return', []);

              case 1:
              case 'end':
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));

      function install() {
        return _ref9.apply(this, arguments);
      }

      return install;
    }()
  }, {
    key: 'process',
    value: function () {
      var _ref10 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9() {
        return _regenerator2.default.wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                if (this.generated) {
                  _context9.next = 11;
                  break;
                }

                _context9.next = 3;
                return this.loadIfNeeded();

              case 3:
                _context9.next = 5;
                return this.pretransform();

              case 5:
                _context9.next = 7;
                return this.getDependencies();

              case 7:
                _context9.next = 9;
                return this.transform();

              case 9:
                this.generated = this.generate();
                this.hash = this.generateHash();

              case 11:
                return _context9.abrupt('return', this.generated);

              case 12:
              case 'end':
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));

      function process() {
        return _ref10.apply(this, arguments);
      }

      return process;
    }()
  }, {
    key: 'generateHash',
    value: function generateHash() {
      return objectHash(this.generated);
    }
  }, {
    key: 'invalidate',
    value: function invalidate() {
      this.processed = false;
      this.contents = this.options ? this.options.contents : null;
      this.ast = null;
      this.generated = null;
      this.hash = null;
      this.dependencies.clear();
      this.depAssets.clear();
    }
  }, {
    key: 'invalidateBundle',
    value: function invalidateBundle() {
      this.parentBundle = null;
      this.bundles.clear();
      this.parentDeps.clear();
    }
  }, {
    key: 'generateBundleName',
    value: function generateBundleName() {
      // Resolve the main file of the package.json
      var main = this.package && this.package.main ? path.resolve(path.dirname(this.package.pkgfile), this.package.main) : null;
      var ext = '.' + this.type;

      // If this asset is main file of the package, use the sanitized package name
      if (this.name === main) {
        var packageName = sanitizeFilename(this.package.name, {
          replacement: '-'
        });
        return packageName + ext;
      }

      // If this is the entry point of the root bundle, use the original filename
      if (this.name === this.options.mainFile) {
        return path.basename(this.name, path.extname(this.name)) + ext;
      }

      // Otherwise generate a unique name
      return md5(this.name) + ext;
    }
  }, {
    key: 'generateErrorMessage',
    value: function generateErrorMessage(err) {
      return err;
    }
  }]);
  return Asset;
}();

module.exports = Asset;