'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Path = require('path');
var crypto = require('crypto');

/**
 * A Bundle represents an output file, containing multiple assets. Bundles can have
 * child bundles, which are bundles that are loaded dynamically from this bundle.
 * Child bundles are also produced when importing an asset of a different type from
 * the bundle, e.g. importing a CSS file from JS.
 */

var Bundle = function () {
  function Bundle(type, name, parent) {
    (0, _classCallCheck3.default)(this, Bundle);

    this.type = type;
    this.name = name;
    this.parentBundle = parent;
    this.entryAsset = null;
    this.assets = new _set2.default();
    this.childBundles = new _set2.default();
    this.siblingBundles = new _set2.default();
    this.siblingBundlesMap = new _map2.default();
  }

  (0, _createClass3.default)(Bundle, [{
    key: 'addAsset',
    value: function addAsset(asset) {
      asset.bundles.add(this);
      this.assets.add(asset);
    }
  }, {
    key: 'removeAsset',
    value: function removeAsset(asset) {
      asset.bundles.delete(this);
      this.assets.delete(asset);
    }
  }, {
    key: 'getSiblingBundle',
    value: function getSiblingBundle(type) {
      if (!type || type === this.type) {
        return this;
      }

      if (!this.siblingBundlesMap.has(type)) {
        var bundle = new Bundle(type, Path.join(Path.dirname(this.name), Path.basename(this.name, Path.extname(this.name)) + '.' + type), this);

        this.childBundles.add(bundle);
        this.siblingBundles.add(bundle);
        this.siblingBundlesMap.set(type, bundle);
      }

      return this.siblingBundlesMap.get(type);
    }
  }, {
    key: 'createChildBundle',
    value: function createChildBundle(entryAsset) {
      var bundle = Bundle.createWithAsset(entryAsset, this);
      this.childBundles.add(bundle);
      return bundle;
    }
  }, {
    key: 'createSiblingBundle',
    value: function createSiblingBundle(entryAsset) {
      var bundle = this.createChildBundle(entryAsset);
      this.siblingBundles.add(bundle);
      return bundle;
    }
  }, {
    key: 'package',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(bundler, oldHashes) {
        var newHashes = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : new _map2.default();

        var hash, promises, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, bundle;

        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!this.isEmpty) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt('return', newHashes);

              case 2:
                hash = this.getHash();

                newHashes.set(this.name, hash);

                promises = [];

                if (!oldHashes || oldHashes.get(this.name) !== hash) {
                  promises.push(this._package(bundler));
                }

                _iteratorNormalCompletion = true;
                _didIteratorError = false;
                _iteratorError = undefined;
                _context.prev = 9;
                for (_iterator = (0, _getIterator3.default)(this.childBundles.values()); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                  bundle = _step.value;

                  promises.push(bundle.package(bundler, oldHashes, newHashes));
                }

                _context.next = 17;
                break;

              case 13:
                _context.prev = 13;
                _context.t0 = _context['catch'](9);
                _didIteratorError = true;
                _iteratorError = _context.t0;

              case 17:
                _context.prev = 17;
                _context.prev = 18;

                if (!_iteratorNormalCompletion && _iterator.return) {
                  _iterator.return();
                }

              case 20:
                _context.prev = 20;

                if (!_didIteratorError) {
                  _context.next = 23;
                  break;
                }

                throw _iteratorError;

              case 23:
                return _context.finish(20);

              case 24:
                return _context.finish(17);

              case 25:
                _context.next = 27;
                return _promise2.default.all(promises);

              case 27:
                return _context.abrupt('return', newHashes);

              case 28:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this, [[9, 13, 17, 25], [18,, 20, 24]]);
      }));

      function _package(_x2, _x3) {
        return _ref.apply(this, arguments);
      }

      return _package;
    }()
  }, {
    key: '_package',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(bundler) {
        var Packager, packager, included, _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, asset;

        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                Packager = bundler.packagers.get(this.type);
                packager = new Packager(this, bundler);
                _context2.next = 4;
                return packager.start();

              case 4:
                included = new _set2.default();
                _iteratorNormalCompletion2 = true;
                _didIteratorError2 = false;
                _iteratorError2 = undefined;
                _context2.prev = 8;
                _iterator2 = (0, _getIterator3.default)(this.assets);

              case 10:
                if (_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done) {
                  _context2.next = 17;
                  break;
                }

                asset = _step2.value;
                _context2.next = 14;
                return this._addDeps(asset, packager, included);

              case 14:
                _iteratorNormalCompletion2 = true;
                _context2.next = 10;
                break;

              case 17:
                _context2.next = 23;
                break;

              case 19:
                _context2.prev = 19;
                _context2.t0 = _context2['catch'](8);
                _didIteratorError2 = true;
                _iteratorError2 = _context2.t0;

              case 23:
                _context2.prev = 23;
                _context2.prev = 24;

                if (!_iteratorNormalCompletion2 && _iterator2.return) {
                  _iterator2.return();
                }

              case 26:
                _context2.prev = 26;

                if (!_didIteratorError2) {
                  _context2.next = 29;
                  break;
                }

                throw _iteratorError2;

              case 29:
                return _context2.finish(26);

              case 30:
                return _context2.finish(23);

              case 31:
                _context2.next = 33;
                return packager.end();

              case 33:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[8, 19, 23, 31], [24,, 26, 30]]);
      }));

      function _package(_x4) {
        return _ref2.apply(this, arguments);
      }

      return _package;
    }()
  }, {
    key: '_addDeps',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(asset, packager, included) {
        var _iteratorNormalCompletion3, _didIteratorError3, _iteratorError3, _iterator3, _step3, depAsset;

        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (!(!this.assets.has(asset) || included.has(asset))) {
                  _context3.next = 2;
                  break;
                }

                return _context3.abrupt('return');

              case 2:

                included.add(asset);

                _iteratorNormalCompletion3 = true;
                _didIteratorError3 = false;
                _iteratorError3 = undefined;
                _context3.prev = 6;
                _iterator3 = (0, _getIterator3.default)(asset.depAssets.values());

              case 8:
                if (_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done) {
                  _context3.next = 15;
                  break;
                }

                depAsset = _step3.value;
                _context3.next = 12;
                return this._addDeps(depAsset, packager, included);

              case 12:
                _iteratorNormalCompletion3 = true;
                _context3.next = 8;
                break;

              case 15:
                _context3.next = 21;
                break;

              case 17:
                _context3.prev = 17;
                _context3.t0 = _context3['catch'](6);
                _didIteratorError3 = true;
                _iteratorError3 = _context3.t0;

              case 21:
                _context3.prev = 21;
                _context3.prev = 22;

                if (!_iteratorNormalCompletion3 && _iterator3.return) {
                  _iterator3.return();
                }

              case 24:
                _context3.prev = 24;

                if (!_didIteratorError3) {
                  _context3.next = 27;
                  break;
                }

                throw _iteratorError3;

              case 27:
                return _context3.finish(24);

              case 28:
                return _context3.finish(21);

              case 29:
                _context3.next = 31;
                return packager.addAsset(asset);

              case 31:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this, [[6, 17, 21, 29], [22,, 24, 28]]);
      }));

      function _addDeps(_x5, _x6, _x7) {
        return _ref3.apply(this, arguments);
      }

      return _addDeps;
    }()
  }, {
    key: 'getParents',
    value: function getParents() {
      var parents = [];
      var bundle = this;

      while (bundle) {
        parents.push(bundle);
        bundle = bundle.parentBundle;
      }

      return parents;
    }
  }, {
    key: 'findCommonAncestor',
    value: function findCommonAncestor(bundle) {
      // Get a list of parent bundles going up to the root
      var ourParents = this.getParents();
      var theirParents = bundle.getParents();

      // Start from the root bundle, and find the first bundle that's different
      var a = ourParents.pop();
      var b = theirParents.pop();
      var last = void 0;
      while (a === b && ourParents.length > 0 && theirParents.length > 0) {
        last = a;
        a = ourParents.pop();
        b = theirParents.pop();
      }

      if (a === b) {
        // One bundle descended from the other
        return a;
      }

      return last;
    }
  }, {
    key: 'getHash',
    value: function getHash() {
      var hash = crypto.createHash('md5');
      var _iteratorNormalCompletion4 = true;
      var _didIteratorError4 = false;
      var _iteratorError4 = undefined;

      try {
        for (var _iterator4 = (0, _getIterator3.default)(this.assets), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
          var asset = _step4.value;

          hash.update(asset.hash);
        }
      } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion4 && _iterator4.return) {
            _iterator4.return();
          }
        } finally {
          if (_didIteratorError4) {
            throw _iteratorError4;
          }
        }
      }

      return hash.digest('hex');
    }
  }, {
    key: 'isEmpty',
    get: function get() {
      return this.assets.size === 0;
    }
  }], [{
    key: 'createWithAsset',
    value: function createWithAsset(asset, parentBundle) {
      var bundle = new Bundle(asset.type, Path.join(asset.options.outDir, asset.generateBundleName()), parentBundle);

      bundle.entryAsset = asset;
      bundle.addAsset(asset);
      return bundle;
    }
  }]);
  return Bundle;
}();

module.exports = Bundle;