'use strict';

var _from = require('babel-runtime/core-js/array/from');

var _from2 = _interopRequireDefault(_from);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var dbg = require('debug')('parcel:Bundler');
var fs = require('./utils/fs');
var Resolver = require('./Resolver');
var Parser = require('./Parser');
var WorkerFarm = require('./WorkerFarm');
var Path = require('path');
var Bundle = require('./Bundle');

var _require = require('chokidar'),
    FSWatcher = _require.FSWatcher;

var FSCache = require('./FSCache');
var HMRServer = require('./HMRServer');
var Server = require('./Server');

var _require2 = require('events'),
    EventEmitter = _require2.EventEmitter;

var Logger = require('./Logger');
var PackagerRegistry = require('./packagers');
var localRequire = require('./utils/localRequire');
var localResolve = require('./utils/localRequire').localResolve;
var config = require('./utils/config');
var emoji = require('./utils/emoji');
var loadEnv = require('./utils/env');
var PromiseQueue = require('./utils/PromiseQueue');

/**
 * The Bundler is the main entry point. It resolves and loads assets,
 * creates the bundle tree, and manages the worker farm, cache, and file watcher.
 */

var Bundler = function (_EventEmitter) {
  (0, _inherits3.default)(Bundler, _EventEmitter);

  function Bundler(main) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    (0, _classCallCheck3.default)(this, Bundler);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Bundler.__proto__ || (0, _getPrototypeOf2.default)(Bundler)).call(this));

    _this.mainFile = Path.resolve(main || '');
    _this.options = _this.normalizeOptions(options);

    _this.resolver = new Resolver(_this.options);
    _this.parser = new Parser(_this.options);
    _this.packagers = new PackagerRegistry();
    _this.cache = _this.options.cache ? new FSCache(_this.options) : null;
    _this.logger = new Logger(_this.options);
    _this.delegate = options.delegate || {};
    _this.bundleLoaders = {};

    _this.addBundleLoader('wasm', require.resolve('./builtins/loaders/wasm-loader'));
    _this.addBundleLoader('css', require.resolve('./builtins/loaders/css-loader'));
    _this.addBundleLoader('js', require.resolve('./builtins/loaders/js-loader'));

    _this.pending = false;
    _this.loadedAssets = new _map2.default();
    _this.watchedAssets = new _map2.default();
    _this.farm = null;
    _this.watcher = null;
    _this.hmr = null;
    _this.bundleHashes = null;
    _this.errored = false;
    _this.buildQueue = new PromiseQueue(_this.processAsset.bind(_this));
    _this.rebuildTimeout = null;
    return _this;
  }

  (0, _createClass3.default)(Bundler, [{
    key: 'normalizeOptions',
    value: function normalizeOptions(options) {
      var isProduction = options.production || process.env.NODE_ENV === 'production';
      var publicURL = options.publicUrl || options.publicURL || '/' + Path.basename(options.outDir || 'dist');
      var watch = typeof options.watch === 'boolean' ? options.watch : !isProduction;
      return {
        outDir: Path.resolve(options.outDir || 'dist'),
        publicURL: publicURL,
        watch: watch,
        cache: typeof options.cache === 'boolean' ? options.cache : true,
        cacheDir: Path.resolve(options.cacheDir || '.cache'),
        killWorkers: typeof options.killWorkers === 'boolean' ? options.killWorkers : true,
        minify: typeof options.minify === 'boolean' ? options.minify : isProduction,
        hmr: typeof options.hmr === 'boolean' ? options.hmr : watch,
        logLevel: typeof options.logLevel === 'number' ? options.logLevel : 3,
        mainFile: this.mainFile,
        hmrPort: options.hmrPort || 0,
        hmrHostname: options.hmrHostname || ''
      };
    }
  }, {
    key: 'addAssetType',
    value: function addAssetType(extension, path) {
      if (typeof path !== 'string') {
        throw new Error('Asset type should be a module path.');
      }

      if (this.farm) {
        throw new Error('Asset types must be added before bundling.');
      }

      this.parser.registerExtension(extension, path);
    }
  }, {
    key: 'addPackager',
    value: function addPackager(type, packager) {
      if (this.farm) {
        throw new Error('Packagers must be added before bundling.');
      }

      this.packagers.add(type, packager);
    }
  }, {
    key: 'addBundleLoader',
    value: function addBundleLoader(type, path) {
      if (typeof path !== 'string') {
        throw new Error('Bundle loader should be a module path.');
      }

      if (this.farm) {
        throw new Error('Bundle loaders must be added before bundling.');
      }

      this.bundleLoaders[type] = path;
    }
  }, {
    key: 'loadPlugins',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
        var pkg, deps, dep, plugin;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return config.load(this.mainFile, ['package.json']);

              case 2:
                pkg = _context.sent;

                if (pkg) {
                  _context.next = 5;
                  break;
                }

                return _context.abrupt('return');

              case 5:
                _context.prev = 5;
                deps = (0, _assign2.default)({}, pkg.dependencies, pkg.devDependencies);
                _context.t0 = _regenerator2.default.keys(deps);

              case 8:
                if ((_context.t1 = _context.t0()).done) {
                  _context.next = 18;
                  break;
                }

                dep = _context.t1.value;

                if (!dep.startsWith('parcel-plugin-')) {
                  _context.next = 16;
                  break;
                }

                _context.next = 13;
                return localRequire(dep, this.mainFile);

              case 13:
                plugin = _context.sent;
                _context.next = 16;
                return plugin(this);

              case 16:
                _context.next = 8;
                break;

              case 18:
                _context.next = 23;
                break;

              case 20:
                _context.prev = 20;
                _context.t2 = _context['catch'](5);

                this.logger.warn(_context.t2);

              case 23:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this, [[5, 20]]);
      }));

      function loadPlugins() {
        return _ref.apply(this, arguments);
      }

      return loadPlugins;
    }()
  }, {
    key: 'bundle',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
        var _this2 = this;

        var isInitialBundle, startTime, loadedAssets, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, asset, _bundle, buildTime, time;

        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (!this.pending) {
                  _context2.next = 2;
                  break;
                }

                return _context2.abrupt('return', new _promise2.default(function (resolve, reject) {
                  _this2.once('buildEnd', function () {
                    _this2.bundle().then(resolve, reject);
                  });
                }));

              case 2:
                isInitialBundle = !this.mainAsset;
                startTime = Date.now();

                if (!(this.buildQueue.empty && !isInitialBundle)) {
                  _context2.next = 6;
                  break;
                }

                return _context2.abrupt('return', null);

              case 6:

                this.pending = true;
                this.errored = false;

                this.logger.clear();
                this.logger.status(emoji.progress, 'Building...');

                _context2.prev = 10;
                _context2.next = 13;
                return this.start();

              case 13:
                if (!isInitialBundle) {
                  _context2.next = 20;
                  break;
                }

                _context2.next = 16;
                return fs.mkdirp(this.options.outDir);

              case 16:
                _context2.next = 18;
                return this.resolveAsset(this.mainFile);

              case 18:
                this.mainAsset = _context2.sent;

                this.buildQueue.add(this.mainAsset);

              case 20:
                _context2.next = 22;
                return this.buildQueue.run();

              case 22:
                loadedAssets = _context2.sent;


                // Emit an HMR update for any new assets (that don't have a parent bundle yet)
                // plus the asset that actually changed.
                if (this.hmr && !isInitialBundle) {
                  this.hmr.emitUpdate([].concat((0, _toConsumableArray3.default)(this.findOrphanAssets()), (0, _toConsumableArray3.default)(loadedAssets)));
                }

                // Invalidate bundles
                _iteratorNormalCompletion = true;
                _didIteratorError = false;
                _iteratorError = undefined;
                _context2.prev = 27;
                for (_iterator = (0, _getIterator3.default)(this.loadedAssets.values()); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                  asset = _step.value;

                  asset.invalidateBundle();
                }

                // Create a new bundle tree and package everything up.
                _context2.next = 35;
                break;

              case 31:
                _context2.prev = 31;
                _context2.t0 = _context2['catch'](27);
                _didIteratorError = true;
                _iteratorError = _context2.t0;

              case 35:
                _context2.prev = 35;
                _context2.prev = 36;

                if (!_iteratorNormalCompletion && _iterator.return) {
                  _iterator.return();
                }

              case 38:
                _context2.prev = 38;

                if (!_didIteratorError) {
                  _context2.next = 41;
                  break;
                }

                throw _iteratorError;

              case 41:
                return _context2.finish(38);

              case 42:
                return _context2.finish(35);

              case 43:
                _bundle = this.createBundleTree(this.mainAsset);
                _context2.next = 46;
                return _bundle.package(this, this.bundleHashes);

              case 46:
                this.bundleHashes = _context2.sent;


                // Unload any orphaned assets
                this.unloadOrphanedAssets();

                buildTime = Date.now() - startTime;
                time = buildTime < 1000 ? buildTime + 'ms' : (buildTime / 1000).toFixed(2) + 's';

                this.logger.status(emoji.success, 'Built in ' + time + '.', 'green');

                this.emit('bundled', _bundle);
                return _context2.abrupt('return', _bundle);

              case 55:
                _context2.prev = 55;
                _context2.t1 = _context2['catch'](10);

                this.errored = true;
                this.logger.error(_context2.t1);
                if (this.hmr) {
                  this.hmr.emitError(_context2.t1);
                }

                if (process.env.NODE_ENV === 'production') {
                  process.exitCode = 1;
                }

              case 61:
                _context2.prev = 61;

                this.pending = false;
                this.emit('buildEnd');

                // If not in watch mode, stop the worker farm so we don't keep the process running.
                if (!this.watcher && this.options.killWorkers) {
                  this.stop();
                }
                return _context2.finish(61);

              case 66:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[10, 55, 61, 66], [27, 31, 35, 43], [36,, 38, 42]]);
      }));

      function bundle() {
        return _ref2.apply(this, arguments);
      }

      return bundle;
    }()
  }, {
    key: 'start',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (!this.farm) {
                  _context3.next = 2;
                  break;
                }

                return _context3.abrupt('return');

              case 2:
                _context3.next = 4;
                return this.loadPlugins();

              case 4:
                _context3.next = 6;
                return loadEnv(this.mainFile);

              case 6:

                this.options.extensions = (0, _assign2.default)({}, this.parser.extensions);
                this.options.bundleLoaders = this.bundleLoaders;
                this.options.env = process.env;

                if (this.options.watch) {
                  // FS events on macOS are flakey in the tests, which write lots of files very quickly
                  // See https://github.com/paulmillr/chokidar/issues/612
                  this.watcher = new FSWatcher({
                    useFsEvents: process.env.NODE_ENV !== 'test'
                  });

                  this.watcher.on('change', this.onChange.bind(this));
                }

                if (!this.options.hmr) {
                  _context3.next = 15;
                  break;
                }

                this.hmr = new HMRServer();
                _context3.next = 14;
                return this.hmr.start(this.options.hmrPort);

              case 14:
                this.options.hmrPort = _context3.sent;

              case 15:

                this.farm = WorkerFarm.getShared(this.options);

              case 16:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function start() {
        return _ref3.apply(this, arguments);
      }

      return start;
    }()
  }, {
    key: 'stop',
    value: function stop() {
      if (this.farm) {
        this.farm.end();
      }

      if (this.watcher) {
        this.watcher.close();
      }

      if (this.hmr) {
        this.hmr.stop();
      }
    }
  }, {
    key: 'getAsset',
    value: function () {
      var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(name, parent) {
        var asset;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return this.resolveAsset(name, parent);

              case 2:
                asset = _context4.sent;

                this.buildQueue.add(asset);
                _context4.next = 6;
                return this.buildQueue.run();

              case 6:
                return _context4.abrupt('return', asset);

              case 7:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function getAsset(_x2, _x3) {
        return _ref4.apply(this, arguments);
      }

      return getAsset;
    }()
  }, {
    key: 'resolveAsset',
    value: function () {
      var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(name, parent) {
        var _this3 = this;

        var isFile, _ref6, path, pkg, asset;

        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                // if (!name.startsWith('/') && !name.startsWith('.') && !name.startsWith('_') && !Path.parse(name).ext) {
                //   dbg('resolveAsset', {name, parent});
                //   let res = await localResolve(name, parent);
                //   dbg('resolveAsset:res', {name, parent, res});
                // }
                isFile = function isFile(file, cb) {
                  if (file === name || _this3.loadedAssets.has(file)) {
                    cb(null, true);
                  } else {
                    // localRequire(name, parent ? Path.parse(parent).dir : parent).then(
                    // try {
                    //   const ext = Path.parse(file).ext;
                    //   if (ext !== 'json') {
                    //     dbg('localResolve', {file, name, parent});
                    //   }
                    //   localResolve(file, parent).then(
                    //     res => {
                    //       if (ext != 'json') {
                    //         dbg('localResolve:res', {file, name, parent, res});
                    //       }
                    //       // return cb(null, !(file instanceof Error))
                    //         fs.stat(res, function (err, stat) {
                    //           // dbg('localResolve:stat', {file, name, err, stat});
                    //           if (err && err.code === 'ENOENT') {
                    //             cb(null, false);
                    //           }
                    //           else if (err) {
                    //             // cb(err);
                    //             cb(null, false);
                    //           }
                    //           else {
                    //             cb(null, stat.isFile() || stat.isFIFO());
                    //           }
                    //         });
                    //     },
                    //     err => {
                    //       // dbg('localResolve:err', {file, name, err});
                    //       // console.log('TKTK');
                    //       // console.error(err);
                    //       return cb(null, false)
                    //     }
                    //   )
                    // } catch (e) {
                    //   dbg('localResolve:error', {name, parent});
                    //   console.log('TKTK err');
                    //   console.error(e);
                    //   return cb(null, false);
                    // }
                    fs.stat(file, function (err, stat) {
                      // dbg('localResolve:stat', {file, name, err, stat});
                      if (err && err.code === 'ENOENT') {
                        cb(null, false);
                      } else if (err) {
                        cb(err);
                        // cb(null, false);
                      } else {
                        cb(null, stat.isFile() || stat.isFIFO());
                      }
                    });
                  }
                };

                _context5.next = 3;
                return this.resolver.resolve(name, parent, { isFile: isFile });

              case 3:
                _ref6 = _context5.sent;
                path = _ref6.path;
                pkg = _ref6.pkg;

                if (!this.loadedAssets.has(path)) {
                  _context5.next = 8;
                  break;
                }

                return _context5.abrupt('return', this.loadedAssets.get(path));

              case 8:
                asset = this.parser.getAsset(path, pkg, this.options);

                this.loadedAssets.set(path, asset);

                this.watch(path, asset);
                return _context5.abrupt('return', asset);

              case 12:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function resolveAsset(_x4, _x5) {
        return _ref5.apply(this, arguments);
      }

      return resolveAsset;
    }()
  }, {
    key: 'watch',
    value: function watch(path, asset) {
      if (!this.watcher) {
        return;
      }

      if (!this.watchedAssets.has(path)) {
        this.watcher.add(path);
        this.watchedAssets.set(path, new _set2.default());
      }

      this.watchedAssets.get(path).add(asset);
    }
  }, {
    key: 'unwatch',
    value: function unwatch(path, asset) {
      if (!this.watchedAssets.has(path)) {
        return;
      }

      var watched = this.watchedAssets.get(path);
      watched.delete(asset);

      if (watched.size === 0) {
        this.watchedAssets.delete(path);
        this.watcher.unwatch(path);
      }
    }
  }, {
    key: 'resolveDep',
    value: function () {
      var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(asset, dep) {
        var thrown, absPath;
        return _regenerator2.default.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                _context6.prev = 0;
                _context6.next = 3;
                return this.resolveAsset(dep.name, asset.name);

              case 3:
                return _context6.abrupt('return', _context6.sent);

              case 6:
                _context6.prev = 6;
                _context6.t0 = _context6['catch'](0);
                thrown = _context6.t0;

                if (!(thrown.message.indexOf('Cannot find module \'' + dep.name + '\'') === 0)) {
                  _context6.next = 18;
                  break;
                }

                thrown.message = 'Cannot resolve dependency \'' + dep.name + '\'';

                // Add absolute path to the error message if the dependency specifies a relative path
                if (dep.name.startsWith('.')) {
                  absPath = Path.resolve(Path.dirname(asset.name), dep.name);

                  _context6.t0.message += ' at \'' + absPath + '\'';
                }

                // Generate a code frame where the dependency was used

                if (!dep.loc) {
                  _context6.next = 17;
                  break;
                }

                _context6.next = 15;
                return asset.loadIfNeeded();

              case 15:
                thrown.loc = dep.loc;
                thrown = asset.generateErrorMessage(thrown);

              case 17:

                thrown.fileName = asset.name;

              case 18:
                throw thrown;

              case 19:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this, [[0, 6]]);
      }));

      function resolveDep(_x6, _x7) {
        return _ref7.apply(this, arguments);
      }

      return resolveDep;
    }()
  }, {
    key: 'processAsset',
    value: function () {
      var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7(asset, isRebuild) {
        return _regenerator2.default.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                if (isRebuild) {
                  asset.invalidate();
                  if (this.cache) {
                    this.cache.invalidate(asset.name);
                  }
                }

                _context7.next = 3;
                return this.loadAsset(asset);

              case 3:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      function processAsset(_x8, _x9) {
        return _ref8.apply(this, arguments);
      }

      return processAsset;
    }()
  }, {
    key: 'loadAsset',
    value: function () {
      var _ref9 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9(asset) {
        var _this4 = this;

        var installDeps, _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, dep, processed, dependencies, implicitDeps, assetDeps;

        return _regenerator2.default.wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                if (!asset.processed) {
                  _context9.next = 2;
                  break;
                }

                return _context9.abrupt('return');

              case 2:

                if (!this.errored) {
                  this.logger.status(emoji.progress, 'Building ' + asset.basename + '...');
                }

                // Mark the asset processed so we don't load it twice
                asset.processed = true;
                _context9.next = 6;
                return asset.install();

              case 6:
                installDeps = _context9.sent;


                this.installed = this.installed || new _set2.default();
                _iteratorNormalCompletion2 = true;
                _didIteratorError2 = false;
                _iteratorError2 = undefined;
                _context9.prev = 11;
                for (_iterator2 = (0, _getIterator3.default)(installDeps); !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                  dep = _step2.value;

                  if (!this.installed.has(dep.name)) {
                    this.installed.add(dep.name);
                    localResolve(dep.name, asset.name);
                  }
                }

                // First try the cache, otherwise load and compile in the background
                _context9.next = 19;
                break;

              case 15:
                _context9.prev = 15;
                _context9.t0 = _context9['catch'](11);
                _didIteratorError2 = true;
                _iteratorError2 = _context9.t0;

              case 19:
                _context9.prev = 19;
                _context9.prev = 20;

                if (!_iteratorNormalCompletion2 && _iterator2.return) {
                  _iterator2.return();
                }

              case 22:
                _context9.prev = 22;

                if (!_didIteratorError2) {
                  _context9.next = 25;
                  break;
                }

                throw _iteratorError2;

              case 25:
                return _context9.finish(22);

              case 26:
                return _context9.finish(19);

              case 27:
                _context9.t1 = this.cache;

                if (!_context9.t1) {
                  _context9.next = 32;
                  break;
                }

                _context9.next = 31;
                return this.cache.read(asset.name);

              case 31:
                _context9.t1 = _context9.sent;

              case 32:
                processed = _context9.t1;

                if (!(!processed || asset.shouldInvalidate(processed.cacheData))) {
                  _context9.next = 38;
                  break;
                }

                _context9.next = 36;
                return this.farm.run(asset.name, asset.contents, asset.package, this.options);

              case 36:
                processed = _context9.sent;

                if (this.cache) {
                  this.cache.write(asset.name, processed);
                }

              case 38:

                asset.generated = processed.generated;
                asset.hash = processed.hash;

                // Call the delegate to get implicit dependencies
                dependencies = processed.dependencies;

                if (!this.delegate.getImplicitDependencies) {
                  _context9.next = 46;
                  break;
                }

                _context9.next = 44;
                return this.delegate.getImplicitDependencies(asset);

              case 44:
                implicitDeps = _context9.sent;

                if (implicitDeps) {
                  dependencies = dependencies.concat(implicitDeps);
                }

              case 46:
                _context9.next = 48;
                return _promise2.default.all(dependencies.map(function () {
                  var _ref10 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8(dep) {
                    var assetDep, _assetDep;

                    return _regenerator2.default.wrap(function _callee8$(_context8) {
                      while (1) {
                        switch (_context8.prev = _context8.next) {
                          case 0:
                            if (!(dep.contents != null)) {
                              _context8.next = 15;
                              break;
                            }

                            assetDep = _this4.loadedAssets.get(dep.name) || _this4.parser.getAsset(dep.name, null, _this4.options, dep.contents);

                            _this4.loadedAssets.set(dep.name, assetDep);
                            _this4.watch(dep.name, assetDep);

                            if (!(assetDep.contents != dep.contents)) {
                              _context8.next = 10;
                              break;
                            }

                            assetDep.options.contents = dep.contents;
                            assetDep.invalidate();
                            if (_this4.cache) {
                              _this4.cache.invalidate(assetDep.name);
                            }
                            _context8.next = 10;
                            return _this4.onChange(assetDep.name);

                          case 10:
                            _context8.next = 12;
                            return _this4.loadAsset(assetDep);

                          case 12:
                            return _context8.abrupt('return', assetDep);

                          case 15:
                            if (!dep.includedInParent) {
                              _context8.next = 19;
                              break;
                            }

                            // This dependency is already included in the parent's generated output,
                            // so no need to load it. We map the name back to the parent asset so
                            // that changing it triggers a recompile of the parent.
                            _this4.watch(dep.name, asset);
                            _context8.next = 26;
                            break;

                          case 19:
                            if (dep.install) {
                              _context8.next = 26;
                              break;
                            }

                            _context8.next = 22;
                            return _this4.resolveDep(asset, dep);

                          case 22:
                            _assetDep = _context8.sent;
                            _context8.next = 25;
                            return _this4.loadAsset(_assetDep);

                          case 25:
                            return _context8.abrupt('return', _assetDep);

                          case 26:
                          case 'end':
                            return _context8.stop();
                        }
                      }
                    }, _callee8, _this4);
                  }));

                  return function (_x11) {
                    return _ref10.apply(this, arguments);
                  };
                }()));

              case 48:
                assetDeps = _context9.sent;


                // Store resolved assets in their original order
                dependencies.forEach(function (dep, i) {
                  asset.dependencies.set(dep.name, dep);
                  var assetDep = assetDeps[i];
                  if (assetDep) {
                    asset.depAssets.set(dep, assetDep);
                  }
                });

              case 50:
              case 'end':
                return _context9.stop();
            }
          }
        }, _callee9, this, [[11, 15, 19, 27], [20,, 22, 26]]);
      }));

      function loadAsset(_x10) {
        return _ref9.apply(this, arguments);
      }

      return loadAsset;
    }()
  }, {
    key: 'createBundleTree',
    value: function createBundleTree(asset, dep, bundle) {
      var parentBundles = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : new _set2.default();

      if (dep) {
        asset.parentDeps.add(dep);
      }

      if (asset.parentBundle) {
        // If the asset is already in a bundle, it is shared. Move it to the lowest common ancestor.
        if (asset.parentBundle !== bundle) {
          var commonBundle = bundle.findCommonAncestor(asset.parentBundle);
          if (asset.parentBundle !== commonBundle && asset.parentBundle.type === commonBundle.type) {
            this.moveAssetToBundle(asset, commonBundle);
            return;
          }
        } else {
          return;
        }

        // Detect circular bundles
        if (parentBundles.has(asset.parentBundle)) {
          return;
        }
      }

      if (!bundle) {
        // Create the root bundle if it doesn't exist
        bundle = Bundle.createWithAsset(asset);
      } else if (dep && dep.dynamic) {
        // Create a new bundle for dynamic imports
        bundle = bundle.createChildBundle(asset);
      } else if (asset.type && !this.packagers.has(asset.type)) {
        // No packager is available for this asset type. Create a new bundle with only this asset.
        bundle.createSiblingBundle(asset);
      } else {
        // Add the asset to the common bundle of the asset's type
        bundle.getSiblingBundle(asset.type).addAsset(asset);
      }

      // If the asset generated a representation for the parent bundle type, also add it there
      if (asset.generated[bundle.type] != null) {
        bundle.addAsset(asset);
      }

      // Add the asset to sibling bundles for each generated type
      if (asset.type && asset.generated[asset.type]) {
        for (var t in asset.generated) {
          if (asset.generated[t]) {
            bundle.getSiblingBundle(t).addAsset(asset);
          }
        }
      }

      asset.parentBundle = bundle;
      parentBundles.add(bundle);

      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = (0, _getIterator3.default)(asset.depAssets), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var _ref11 = _step3.value;

          var _ref12 = (0, _slicedToArray3.default)(_ref11, 2);

          var _dep = _ref12[0];
          var assetDep = _ref12[1];

          this.createBundleTree(assetDep, _dep, bundle, parentBundles);
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      parentBundles.delete(bundle);
      return bundle;
    }
  }, {
    key: 'moveAssetToBundle',
    value: function moveAssetToBundle(asset, commonBundle) {
      // Never move the entry asset of a bundle, as it was explicitly requested to be placed in a separate bundle.
      if (asset.parentBundle.entryAsset === asset) {
        return;
      }

      var _iteratorNormalCompletion4 = true;
      var _didIteratorError4 = false;
      var _iteratorError4 = undefined;

      try {
        for (var _iterator4 = (0, _getIterator3.default)((0, _from2.default)(asset.bundles)), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
          var bundle = _step4.value;

          bundle.removeAsset(asset);
          commonBundle.getSiblingBundle(bundle.type).addAsset(asset);
        }
      } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion4 && _iterator4.return) {
            _iterator4.return();
          }
        } finally {
          if (_didIteratorError4) {
            throw _iteratorError4;
          }
        }
      }

      var oldBundle = asset.parentBundle;
      asset.parentBundle = commonBundle;

      // Move all dependencies as well
      var _iteratorNormalCompletion5 = true;
      var _didIteratorError5 = false;
      var _iteratorError5 = undefined;

      try {
        for (var _iterator5 = (0, _getIterator3.default)(asset.depAssets.values()), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
          var child = _step5.value;

          if (child.parentBundle === oldBundle) {
            this.moveAssetToBundle(child, commonBundle);
          }
        }
      } catch (err) {
        _didIteratorError5 = true;
        _iteratorError5 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion5 && _iterator5.return) {
            _iterator5.return();
          }
        } finally {
          if (_didIteratorError5) {
            throw _iteratorError5;
          }
        }
      }
    }
  }, {
    key: 'findOrphanAssets',
    value: /*#__PURE__*/_regenerator2.default.mark(function findOrphanAssets() {
      var _iteratorNormalCompletion6, _didIteratorError6, _iteratorError6, _iterator6, _step6, asset;

      return _regenerator2.default.wrap(function findOrphanAssets$(_context10) {
        while (1) {
          switch (_context10.prev = _context10.next) {
            case 0:
              _iteratorNormalCompletion6 = true;
              _didIteratorError6 = false;
              _iteratorError6 = undefined;
              _context10.prev = 3;
              _iterator6 = (0, _getIterator3.default)(this.loadedAssets.values());

            case 5:
              if (_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done) {
                _context10.next = 13;
                break;
              }

              asset = _step6.value;

              if (asset.parentBundle) {
                _context10.next = 10;
                break;
              }

              _context10.next = 10;
              return asset;

            case 10:
              _iteratorNormalCompletion6 = true;
              _context10.next = 5;
              break;

            case 13:
              _context10.next = 19;
              break;

            case 15:
              _context10.prev = 15;
              _context10.t0 = _context10['catch'](3);
              _didIteratorError6 = true;
              _iteratorError6 = _context10.t0;

            case 19:
              _context10.prev = 19;
              _context10.prev = 20;

              if (!_iteratorNormalCompletion6 && _iterator6.return) {
                _iterator6.return();
              }

            case 22:
              _context10.prev = 22;

              if (!_didIteratorError6) {
                _context10.next = 25;
                break;
              }

              throw _iteratorError6;

            case 25:
              return _context10.finish(22);

            case 26:
              return _context10.finish(19);

            case 27:
            case 'end':
              return _context10.stop();
          }
        }
      }, findOrphanAssets, this, [[3, 15, 19, 27], [20,, 22, 26]]);
    })
  }, {
    key: 'unloadOrphanedAssets',
    value: function unloadOrphanedAssets() {
      var _iteratorNormalCompletion7 = true;
      var _didIteratorError7 = false;
      var _iteratorError7 = undefined;

      try {
        for (var _iterator7 = (0, _getIterator3.default)(this.findOrphanAssets()), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
          var asset = _step7.value;

          this.unloadAsset(asset);
        }
      } catch (err) {
        _didIteratorError7 = true;
        _iteratorError7 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion7 && _iterator7.return) {
            _iterator7.return();
          }
        } finally {
          if (_didIteratorError7) {
            throw _iteratorError7;
          }
        }
      }
    }
  }, {
    key: 'unloadAsset',
    value: function unloadAsset(asset) {
      this.loadedAssets.delete(asset.name);
      if (this.watcher) {
        this.unwatch(asset.name, asset);

        // Unwatch all included dependencies that map to this asset
        var _iteratorNormalCompletion8 = true;
        var _didIteratorError8 = false;
        var _iteratorError8 = undefined;

        try {
          for (var _iterator8 = (0, _getIterator3.default)(asset.dependencies.values()), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
            var dep = _step8.value;

            if (dep.includedInParent) {
              this.unwatch(dep.name, asset);
            }
          }
        } catch (err) {
          _didIteratorError8 = true;
          _iteratorError8 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion8 && _iterator8.return) {
              _iterator8.return();
            }
          } finally {
            if (_didIteratorError8) {
              throw _iteratorError8;
            }
          }
        }
      }
    }
  }, {
    key: 'onChange',
    value: function () {
      var _ref13 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee11(path) {
        var _this5 = this;

        var assets, _iteratorNormalCompletion9, _didIteratorError9, _iteratorError9, _iterator9, _step9, asset;

        return _regenerator2.default.wrap(function _callee11$(_context12) {
          while (1) {
            switch (_context12.prev = _context12.next) {
              case 0:
                dbg('onChange', path);
                assets = this.watchedAssets.get(path);

                if (!(!assets || !assets.size)) {
                  _context12.next = 4;
                  break;
                }

                return _context12.abrupt('return');

              case 4:

                this.logger.clear();
                this.logger.status(emoji.progress, 'Building ' + Path.basename(path) + '...');

                // Add the asset to the rebuild queue, and reset the timeout.
                _iteratorNormalCompletion9 = true;
                _didIteratorError9 = false;
                _iteratorError9 = undefined;
                _context12.prev = 9;
                for (_iterator9 = (0, _getIterator3.default)(assets); !(_iteratorNormalCompletion9 = (_step9 = _iterator9.next()).done); _iteratorNormalCompletion9 = true) {
                  asset = _step9.value;

                  dbg('onChange:asset', asset.name);
                  this.buildQueue.add(asset, true);
                }

                _context12.next = 17;
                break;

              case 13:
                _context12.prev = 13;
                _context12.t0 = _context12['catch'](9);
                _didIteratorError9 = true;
                _iteratorError9 = _context12.t0;

              case 17:
                _context12.prev = 17;
                _context12.prev = 18;

                if (!_iteratorNormalCompletion9 && _iterator9.return) {
                  _iterator9.return();
                }

              case 20:
                _context12.prev = 20;

                if (!_didIteratorError9) {
                  _context12.next = 23;
                  break;
                }

                throw _iteratorError9;

              case 23:
                return _context12.finish(20);

              case 24:
                return _context12.finish(17);

              case 25:
                clearTimeout(this.rebuildTimeout);

                this.rebuildTimeout = setTimeout((0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee10() {
                  return _regenerator2.default.wrap(function _callee10$(_context11) {
                    while (1) {
                      switch (_context11.prev = _context11.next) {
                        case 0:
                          _context11.next = 2;
                          return _this5.bundle();

                        case 2:
                        case 'end':
                          return _context11.stop();
                      }
                    }
                  }, _callee10, _this5);
                })), 100);

              case 27:
              case 'end':
                return _context12.stop();
            }
          }
        }, _callee11, this, [[9, 13, 17, 25], [18,, 20, 24]]);
      }));

      function onChange(_x13) {
        return _ref13.apply(this, arguments);
      }

      return onChange;
    }()
  }, {
    key: 'middleware',
    value: function middleware() {
      this.bundle();
      return Server.middleware(this);
    }
  }, {
    key: 'serve',
    value: function () {
      var _ref15 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee12() {
        var port = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1234;
        var https = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
        var server;
        return _regenerator2.default.wrap(function _callee12$(_context13) {
          while (1) {
            switch (_context13.prev = _context13.next) {
              case 0:
                _context13.next = 2;
                return Server.serve(this, port, https);

              case 2:
                server = _context13.sent;

                this.bundle();
                return _context13.abrupt('return', server);

              case 5:
              case 'end':
                return _context13.stop();
            }
          }
        }, _callee12, this);
      }));

      function serve() {
        return _ref15.apply(this, arguments);
      }

      return serve;
    }()
  }]);
  return Bundler;
}(EventEmitter);

module.exports = Bundler;
Bundler.Asset = require('./Asset');
Bundler.Packager = require('./packagers/Packager');