'use strict';

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var chalk = require('chalk');
var readline = require('readline');
var prettyError = require('./utils/prettyError');
var emoji = require('./utils/emoji');

var Logger = function () {
  function Logger(options) {
    (0, _classCallCheck3.default)(this, Logger);

    this.logLevel = typeof options.logLevel === 'number' ? options.logLevel : 3;
    this.color = typeof options.color === 'boolean' ? options.color : chalk.supportsColor;
    this.chalk = new chalk.constructor({ enabled: this.color });
    this.lines = 0;
    this.statusLine = null;
  }

  (0, _createClass3.default)(Logger, [{
    key: 'write',
    value: function write(message) {
      var persistent = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

      if (!persistent) {
        this.lines += message.split('\n').length;
      }

      console.log(message);
    }
  }, {
    key: 'log',
    value: function log(message) {
      if (this.logLevel < 3) {
        return;
      }

      this.write(message);
    }
  }, {
    key: 'persistent',
    value: function persistent(message) {
      if (this.logLevel < 3) {
        return;
      }

      this.write(this.chalk.bold(message), true);
    }
  }, {
    key: 'warn',
    value: function warn(message) {
      if (this.logLevel < 2) {
        return;
      }

      this.write(this.chalk.yellow(emoji.warning + ' ' + message));
    }
  }, {
    key: 'error',
    value: function error(err) {
      if (this.logLevel < 1) {
        return;
      }

      var _prettyError = prettyError(err, { color: this.color }),
          message = _prettyError.message,
          stack = _prettyError.stack;

      this.status(emoji.error, message, 'red');
      if (stack) {
        this.write(stack);
      }
    }
  }, {
    key: 'clear',
    value: function clear() {
      if (!this.color) {
        return;
      }

      while (this.lines > 0) {
        readline.clearLine(process.stdout, 0);
        readline.moveCursor(process.stdout, 0, -1);
        this.lines--;
      }

      readline.cursorTo(process.stdout, 0);
      this.statusLine = null;
    }
  }, {
    key: 'writeLine',
    value: function writeLine(line, msg) {
      if (!this.color) {
        return this.log(msg);
      }

      var n = this.lines - line;
      var stdout = process.stdout;
      readline.cursorTo(stdout, 0);
      readline.moveCursor(stdout, 0, -n);
      stdout.write(msg);
      readline.clearLine(stdout, 1);
      readline.cursorTo(stdout, 0);
      readline.moveCursor(stdout, 0, n);
    }
  }, {
    key: 'status',
    value: function status(emoji, message) {
      var color = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'gray';

      if (this.logLevel < 3) {
        return;
      }

      var hasStatusLine = this.statusLine != null;
      if (!hasStatusLine) {
        this.statusLine = this.lines;
      }

      this.writeLine(this.statusLine, this.chalk[color].bold(emoji + '  ' + message));

      if (!hasStatusLine) {
        process.stdout.write('\n');
        this.lines++;
      }
    }
  }]);
  return Logger;
}();

module.exports = Logger;