'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var serve = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(bundler, port) {
    var useHTTPS = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    var handler, server, freePort;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            handler = middleware(bundler);
            server = useHTTPS ? https.createServer(generateCertificate(bundler.options), handler) : http.createServer(handler);
            _context.next = 4;
            return getPort({ port: port });

          case 4:
            freePort = _context.sent;

            server.listen(freePort);

            return _context.abrupt('return', new _promise2.default(function (resolve, reject) {
              server.on('error', function (err) {
                bundler.logger.error(new Error(serverErrors(err, server.address().port)));
                reject(err);
              });

              server.once('listening', function () {
                var addon = server.address().port !== port ? '- ' + bundler.logger.chalk.yellow('configured port ' + port + ' could not be used.') : '';

                bundler.logger.persistent('Server running at ' + bundler.logger.chalk.cyan((useHTTPS ? 'https' : 'http') + '://localhost:' + server.address().port) + ' ' + addon);

                resolve(server);
              });
            }));

          case 7:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function serve(_x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var http = require('http');
var https = require('https');
var serveStatic = require('serve-static');
var getPort = require('get-port');
var serverErrors = require('./utils/customErrors').serverErrors;
var generateCertificate = require('./utils/generateCertificate');

serveStatic.mime.define({
  'application/wasm': ['wasm']
});

function middleware(bundler) {
  var serve = serveStatic(bundler.options.outDir, { index: false });

  return function (req, res, next) {
    // Wait for the bundler to finish bundling if needed
    if (bundler.pending) {
      bundler.once('bundled', respond);
    } else {
      respond();
    }

    function respond() {
      if (bundler.errored) {
        return send500();
      } else if (!req.url.startsWith(bundler.options.publicURL)) {
        // If the URL doesn't start with the public path, send the main HTML bundle
        return sendIndex();
      } else {
        // Otherwise, serve the file from the dist folder
        req.url = req.url.slice(bundler.options.publicURL.length);
        return serve(req, res, send404);
      }
    }

    function sendIndex() {
      // If the main asset is an HTML file, serve it
      if (bundler.mainAsset.type === 'html') {
        req.url = '/' + bundler.mainAsset.generateBundleName();
        serve(req, res, send404);
      } else {
        send404();
      }
    }

    function send500() {
      res.setHeader('Content-Type', 'text/plain; charset=utf-8');
      res.writeHead(500);
      res.end('🚨 Build error, check the console for details.');
    }

    function send404() {
      if (next) {
        return next();
      }

      res.writeHead(404);
      res.end();
    }
  };
}

exports.middleware = middleware;
exports.serve = serve;