'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _require = require('source-map'),
    SourceMapConsumer = _require.SourceMapConsumer,
    SourceMapGenerator = _require.SourceMapGenerator;

var lineCounter = require('./utils/lineCounter');

var SourceMap = function () {
  function SourceMap(mappings, sources) {
    (0, _classCallCheck3.default)(this, SourceMap);

    this.mappings = this.purifyMappings(mappings);
    this.sources = sources || {};
    this.lineCount = null;
  }

  (0, _createClass3.default)(SourceMap, [{
    key: 'purifyMappings',
    value: function purifyMappings(mappings) {
      if (Array.isArray(mappings)) {
        return mappings.filter(function (mapping) {
          return mapping && mapping.source && mapping.original && typeof mapping.original.line === 'number' && mapping.original.line > 0 && typeof mapping.original.column === 'number' && mapping.generated && typeof mapping.generated.line === 'number' && mapping.generated.line > 0 && typeof mapping.generated.column === 'number';
        });
      }

      return [];
    }
  }, {
    key: 'getConsumer',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(map) {
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!(map instanceof SourceMapConsumer)) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt('return', map);

              case 2:
                map = typeof map === 'string' ? JSON.parse(map) : map;
                _context.next = 5;
                return new SourceMapConsumer(map);

              case 5:
                return _context.abrupt('return', _context.sent);

              case 6:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function getConsumer(_x) {
        return _ref.apply(this, arguments);
      }

      return getConsumer;
    }()
  }, {
    key: 'addMap',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(map) {
        var _this = this;

        var lineOffset = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
        var columnOffset = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
        var consumer;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (!(!(map instanceof SourceMap) && map.version)) {
                  _context2.next = 8;
                  break;
                }

                _context2.next = 3;
                return this.getConsumer(map);

              case 3:
                consumer = _context2.sent;


                consumer.eachMapping(function (mapping) {
                  _this.addConsumerMapping(mapping, lineOffset, columnOffset);
                  if (!_this.sources[mapping.source]) {
                    _this.sources[mapping.source] = consumer.sourceContentFor(mapping.source, true);
                  }
                });

                if (consumer.destroy) {
                  // Only needs to happen in source-map 0.7
                  consumer.destroy();
                }
                _context2.next = 11;
                break;

              case 8:
                if (!map.eachMapping) {
                  map = new SourceMap(map.mappings, map.sources);
                }

                if (lineOffset === 0 && columnOffset === 0) {
                  this.mappings = this.mappings.concat(map.mappings);
                } else {
                  map.eachMapping(function (mapping) {
                    _this.addMapping(mapping, lineOffset, columnOffset);
                  });
                }

                (0, _keys2.default)(map.sources).forEach(function (sourceName) {
                  if (!_this.sources[sourceName]) {
                    _this.sources[sourceName] = map.sources[sourceName];
                  }
                });

              case 11:
                return _context2.abrupt('return', this);

              case 12:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function addMap(_x4) {
        return _ref2.apply(this, arguments);
      }

      return addMap;
    }()
  }, {
    key: 'addMapping',
    value: function addMapping(mapping) {
      var lineOffset = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var columnOffset = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

      mapping.generated = {
        line: mapping.generated.line + lineOffset,
        column: mapping.generated.column + columnOffset
      };

      this.mappings.push(mapping);
    }
  }, {
    key: 'addConsumerMapping',
    value: function addConsumerMapping(mapping) {
      var lineOffset = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var columnOffset = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

      if (!mapping.source || !mapping.originalLine || !mapping.originalColumn && mapping.originalColumn !== 0) {
        return;
      }

      this.mappings.push({
        source: mapping.source,
        original: {
          line: mapping.originalLine,
          column: mapping.originalColumn
        },
        generated: {
          line: mapping.generatedLine + lineOffset,
          column: mapping.generatedColumn + columnOffset
        },
        name: mapping.name
      });
    }
  }, {
    key: 'eachMapping',
    value: function eachMapping(callback) {
      this.mappings.forEach(callback);
    }
  }, {
    key: 'generateEmptyMap',
    value: function generateEmptyMap(sourceName, sourceContent) {
      this.sources[sourceName] = sourceContent;

      this.lineCount = lineCounter(sourceContent);
      for (var line = 1; line < this.lineCount + 1; line++) {
        this.addMapping({
          source: sourceName,
          original: {
            line: line,
            column: 0
          },
          generated: {
            line: line,
            column: 0
          }
        });
      }

      return this;
    }
  }, {
    key: 'extendSourceMap',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(original, extension) {
        var _this2 = this;

        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (extension instanceof SourceMap) {
                  _context3.next = 2;
                  break;
                }

                throw new Error('[SOURCEMAP] Type of extension should be a SourceMap instance!');

              case 2:
                _context3.next = 4;
                return this.getConsumer(original);

              case 4:
                original = _context3.sent;

                extension.eachMapping(function (mapping) {
                  var originalMapping = original.originalPositionFor({
                    line: mapping.original.line,
                    column: mapping.original.column
                  });

                  if (!originalMapping.line) {
                    return false;
                  }

                  _this2.addMapping({
                    source: originalMapping.source,
                    name: originalMapping.name,
                    original: {
                      line: originalMapping.line,
                      column: originalMapping.column
                    },
                    generated: {
                      line: mapping.generated.line,
                      column: mapping.generated.column
                    }
                  });

                  if (!_this2.sources[originalMapping.source]) {
                    _this2.sources[originalMapping.source] = original.sourceContentFor(originalMapping.source, true);
                  }
                });

                if (original.destroy) {
                  // Only needs to happen in source-map 0.7
                  original.destroy();
                }

                return _context3.abrupt('return', this);

              case 8:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function extendSourceMap(_x9, _x10) {
        return _ref3.apply(this, arguments);
      }

      return extendSourceMap;
    }()
  }, {
    key: 'offset',
    value: function offset() {
      var lineOffset = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      var columnOffset = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

      this.mappings.map(function (mapping) {
        mapping.generated.line = mapping.generated.line + lineOffset;
        mapping.generated.column = mapping.generated.column + columnOffset;
        return mapping;
      });

      if (this.lineCount != null) {
        this.lineCount += lineOffset;
      }
    }
  }, {
    key: 'stringify',
    value: function stringify(file) {
      var _this3 = this;

      var generator = new SourceMapGenerator({
        file: file
      });

      this.eachMapping(function (mapping) {
        return generator.addMapping(mapping);
      });
      (0, _keys2.default)(this.sources).forEach(function (sourceName) {
        return generator.setSourceContent(sourceName, _this3.sources[sourceName]);
      });

      return generator.toString();
    }
  }]);
  return SourceMap;
}();

module.exports = SourceMap;