'use strict';

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _get2 = require('babel-runtime/helpers/get');

var _get3 = _interopRequireDefault(_get2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var path = require('path');
var commandExists = require('command-exists');
var childProcess = require('child_process');
var promisify = require('../utils/promisify');
var exec = promisify(childProcess.execFile);
var tomlify = require('tomlify-j0.4');
var fs = require('../utils/fs');
var Asset = require('../Asset');
var config = require('../utils/config');
var pipeSpawn = require('../utils/pipeSpawn');
var md5 = require('../utils/md5');

var RUST_TARGET = 'wasm32-unknown-unknown';
var MAIN_FILES = ['src/lib.rs', 'src/main.rs'];

// Track installation status so we don't need to check more than once
var rustInstalled = false;
var wasmGCInstalled = false;

var RustAsset = function (_Asset) {
  (0, _inherits3.default)(RustAsset, _Asset);

  function RustAsset(name, pkg, options) {
    (0, _classCallCheck3.default)(this, RustAsset);

    var _this = (0, _possibleConstructorReturn3.default)(this, (RustAsset.__proto__ || (0, _getPrototypeOf2.default)(RustAsset)).call(this, name, pkg, options));

    _this.type = 'wasm';
    return _this;
  }

  (0, _createClass3.default)(RustAsset, [{
    key: 'process',
    value: function process() {
      // We don't want to process this asset if the worker is in a warm up phase
      // since the asset will also be processed by the main process, which
      // may cause errors since rust writes to the filesystem.
      if (this.options.isWarmUp) {
        return;
      }

      return (0, _get3.default)(RustAsset.prototype.__proto__ || (0, _getPrototypeOf2.default)(RustAsset.prototype), 'process', this).call(this);
    }
  }, {
    key: 'parse',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
        var _this2 = this;

        var cargoConfig, cargoDir, isMainFile, mainFiles;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return this.installRust();

              case 2:
                _context.next = 4;
                return this.getConfig(['Cargo.toml']);

              case 4:
                cargoConfig = _context.sent;
                cargoDir = void 0;
                isMainFile = false;

                if (!cargoConfig) {
                  _context.next = 16;
                  break;
                }

                mainFiles = MAIN_FILES.slice();

                if (cargoConfig.lib && cargoConfig.lib.path) {
                  mainFiles.push(cargoConfig.lib.path);
                }

                _context.t0 = path;
                _context.next = 13;
                return config.resolve(this.name, ['Cargo.toml']);

              case 13:
                _context.t1 = _context.sent;
                cargoDir = _context.t0.dirname.call(_context.t0, _context.t1);

                isMainFile = mainFiles.some(function (file) {
                  return path.join(cargoDir, file) === _this2.name;
                });

              case 16:
                if (!isMainFile) {
                  _context.next = 21;
                  break;
                }

                _context.next = 19;
                return this.cargoBuild(cargoConfig, cargoDir);

              case 19:
                _context.next = 23;
                break;

              case 21:
                _context.next = 23;
                return this.rustcBuild();

              case 23:
                if (!this.options.minify) {
                  _context.next = 28;
                  break;
                }

                _context.next = 26;
                return this.installWasmGC();

              case 26:
                _context.next = 28;
                return exec('wasm-gc', [this.wasmPath, this.wasmPath]);

              case 28:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function parse() {
        return _ref.apply(this, arguments);
      }

      return parse;
    }()
  }, {
    key: 'installRust',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
        var _ref3, _ref4, stdout, _ref5, _ref6;

        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (!rustInstalled) {
                  _context2.next = 2;
                  break;
                }

                return _context2.abrupt('return');

              case 2:
                _context2.prev = 2;
                _context2.next = 5;
                return commandExists('rustup');

              case 5:
                _context2.next = 10;
                break;

              case 7:
                _context2.prev = 7;
                _context2.t0 = _context2['catch'](2);
                throw new Error("Rust isn't installed. Visit https://www.rustup.rs/ for more info");

              case 10:
                _context2.next = 12;
                return exec('rustup', ['show']);

              case 12:
                _ref3 = _context2.sent;
                _ref4 = (0, _slicedToArray3.default)(_ref3, 1);
                stdout = _ref4[0];

                if (stdout.includes('nightly')) {
                  _context2.next = 20;
                  break;
                }

                _context2.next = 18;
                return pipeSpawn('rustup', ['update']);

              case 18:
                _context2.next = 20;
                return pipeSpawn('rustup', ['toolchain', 'install', 'nightly']);

              case 20:
                _context2.next = 22;
                return exec('rustup', ['target', 'list', '--toolchain', 'nightly']);

              case 22:
                _ref5 = _context2.sent;
                _ref6 = (0, _slicedToArray3.default)(_ref5, 1);
                stdout = _ref6[0];

                if (stdout.includes(RUST_TARGET + ' (installed)')) {
                  _context2.next = 28;
                  break;
                }

                _context2.next = 28;
                return pipeSpawn('rustup', ['target', 'add', RUST_TARGET, '--toolchain', 'nightly']);

              case 28:

                rustInstalled = true;

              case 29:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[2, 7]]);
      }));

      function installRust() {
        return _ref2.apply(this, arguments);
      }

      return installRust;
    }()
  }, {
    key: 'installWasmGC',
    value: function () {
      var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (!wasmGCInstalled) {
                  _context3.next = 2;
                  break;
                }

                return _context3.abrupt('return');

              case 2:
                _context3.prev = 2;
                _context3.next = 5;
                return commandExists('wasm-gc');

              case 5:
                _context3.next = 11;
                break;

              case 7:
                _context3.prev = 7;
                _context3.t0 = _context3['catch'](2);
                _context3.next = 11;
                return pipeSpawn('cargo', ['install', '--git', 'https://github.com/alexcrichton/wasm-gc']);

              case 11:

                wasmGCInstalled = true;

              case 12:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this, [[2, 7]]);
      }));

      function installWasmGC() {
        return _ref7.apply(this, arguments);
      }

      return installWasmGC;
    }()
  }, {
    key: 'cargoBuild',
    value: function () {
      var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(cargoConfig, cargoDir) {
        var args, outDir, rustName;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                // Ensure the cargo config has cdylib as the crate-type
                if (!cargoConfig.lib) {
                  cargoConfig.lib = {};
                }

                if (!Array.isArray(cargoConfig.lib['crate-type'])) {
                  cargoConfig.lib['crate-type'] = [];
                }

                if (cargoConfig.lib['crate-type'].includes('cdylib')) {
                  _context4.next = 6;
                  break;
                }

                cargoConfig.lib['crate-type'].push('cdylib');
                _context4.next = 6;
                return fs.writeFile(path.join(cargoDir, 'Cargo.toml'), tomlify.toToml(cargoConfig));

              case 6:

                // Run cargo
                args = ['+nightly', 'build', '--target', RUST_TARGET, '--release'];
                _context4.next = 9;
                return exec('cargo', args, { cwd: cargoDir });

              case 9:

                // Get output file paths
                outDir = path.join(cargoDir, 'target', RUST_TARGET, 'release');

                // Rust converts '-' to '_' when outputting files.

                rustName = cargoConfig.package.name.replace(/-/g, '_');

                this.wasmPath = path.join(outDir, rustName + '.wasm');
                this.depsPath = path.join(outDir, rustName + '.d');

              case 13:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function cargoBuild(_x, _x2) {
        return _ref8.apply(this, arguments);
      }

      return cargoBuild;
    }()
  }, {
    key: 'rustcBuild',
    value: function () {
      var _ref9 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5() {
        var name, args;
        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _context5.next = 2;
                return fs.mkdirp(this.options.cacheDir);

              case 2:
                name = md5(this.name);

                this.wasmPath = path.join(this.options.cacheDir, name + '.wasm');

                // Run rustc to compile the code
                args = ['+nightly', '--target', RUST_TARGET, '-O', '--crate-type=cdylib', this.name, '-o', this.wasmPath];
                _context5.next = 7;
                return exec('rustc', args);

              case 7:

                // Run again to collect dependencies
                this.depsPath = path.join(this.options.cacheDir, name + '.d');
                _context5.next = 10;
                return exec('rustc', [this.name, '--emit=dep-info', '-o', this.depsPath]);

              case 10:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function rustcBuild() {
        return _ref9.apply(this, arguments);
      }

      return rustcBuild;
    }()
  }, {
    key: 'collectDependencies',
    value: function () {
      var _ref10 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6() {
        var contents, dir, deps, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, dep;

        return _regenerator2.default.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                _context6.next = 2;
                return fs.readFile(this.depsPath, 'utf8');

              case 2:
                contents = _context6.sent;
                dir = path.dirname(this.name);
                deps = contents.split('\n').filter(Boolean).slice(1);
                _iteratorNormalCompletion = true;
                _didIteratorError = false;
                _iteratorError = undefined;
                _context6.prev = 8;


                for (_iterator = (0, _getIterator3.default)(deps); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                  dep = _step.value;

                  dep = path.resolve(dir, dep.slice(0, dep.indexOf(':')));
                  if (dep !== this.name) {
                    this.addDependency(dep, { includedInParent: true });
                  }
                }
                _context6.next = 16;
                break;

              case 12:
                _context6.prev = 12;
                _context6.t0 = _context6['catch'](8);
                _didIteratorError = true;
                _iteratorError = _context6.t0;

              case 16:
                _context6.prev = 16;
                _context6.prev = 17;

                if (!_iteratorNormalCompletion && _iterator.return) {
                  _iterator.return();
                }

              case 19:
                _context6.prev = 19;

                if (!_didIteratorError) {
                  _context6.next = 22;
                  break;
                }

                throw _iteratorError;

              case 22:
                return _context6.finish(19);

              case 23:
                return _context6.finish(16);

              case 24:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this, [[8, 12, 16, 24], [17,, 19, 23]]);
      }));

      function collectDependencies() {
        return _ref10.apply(this, arguments);
      }

      return collectDependencies;
    }()
  }, {
    key: 'generate',
    value: function () {
      var _ref11 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7() {
        return _regenerator2.default.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                return _context7.abrupt('return', {
                  wasm: {
                    path: this.wasmPath, // pass output path to RawPackager
                    mtime: Date.now() // force re-bundling since otherwise the hash would never change
                  }
                });

              case 1:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      function generate() {
        return _ref11.apply(this, arguments);
      }

      return generate;
    }()
  }]);
  return RustAsset;
}(Asset);

module.exports = RustAsset;