'use strict';

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var compiler = require('vue-component-compiler');

// utility for generating a uid for each component file
// used in scoped CSS rewriting
var hash = require('hash-sum');
var cache = (0, _create2.default)(null);

var genId = function genId(file) {
  return cache[file] || (cache[file] = hash(file));
};

module.exports = function compile(code, name, opts) {
  var dependencies = new _map2.default();
  var id = genId(name);
  var scopeId = 'data-v-' + id;
  // const id = 'xxx';
  var descriptors = compiler.parse(code, name, opts || { needMap: false });
  var source = {
    styles: [
      // { id: cssid, descriptor: descriptors.styles[0] },
    ],
    customBlocks: [
      // { id: '?type=custom&index=0', descriptor: descriptors.customBlocks[0] }
    ]
  };

  {
    var desc = descriptors.template || { content: '' };
    var template = compiler.compileTemplate({ code: desc.content, descriptor: {} }, name, {
      scopeId: scopeId,
      options: { scopeId: scopeId },
      esModule: true,
      isProduction: false,
      isHot: true
    });
    dependencies.set(name + '__type=template.js', {
      code: template.code,
      descriptor: desc
    });
    source.render = { id: name + '__type=template.js', descriptor: desc };
  }

  if (descriptors.script) {
    var _ext = '.' + (descriptors.script.lang || 'js');
    source.script = {
      id: name + '__type=script' + _ext,
      descriptor: descriptors.script || {}
    };
  } else {
    source.script = { id: name + '__type=script.js', descriptor: {} };
  }
  var ext = '.' + ((descriptors.script || {}).lang || 'js');
  dependencies.set(name + '__type=script' + ext, {
    code: descriptors.script ? descriptors.script.content : ' ',
    descriptor: descriptors.script || {}
  });

  for (var i = 0; i < descriptors.styles.length; i++) {
    var cssid = name + '__type=style&index=' + i + '.' + (typeof descriptors.styles[i].lang === 'string' ? descriptors.styles[i].lang : 'css');
    source.styles.push({ id: cssid, descriptor: descriptors.styles[i] });
    var style = descriptors.styles[i].lang ? { code: descriptors.styles[i].content } : compiler.compileStyle({
      code: descriptors.styles[i].content,
      descriptor: (0, _assign2.default)({}, descriptors.styles[i].attrs, descriptors.styles[i])
    }, name, {
      scopeId: scopeId,
      needMap: false,
      async: false
    });
    dependencies.set(cssid, (0, _assign2.default)({ descriptor: descriptors.styles[i] }, style));
  }

  var result = {
    code: compiler.assemble(source, name, {
      scopeId: scopeId,
      moduleId: scopeId,
      isServer: false,
      isHot: true,
      hasStyleInjectFn: true,
      esModule: true,
      isProduction: false
    })
  };
  result.dependencies = dependencies;
  return result;
};

module.exports.compiler = compiler;
module.exports.genId = genId;