'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _from = require('babel-runtime/core-js/array/from');

var _from2 = _interopRequireDefault(_from);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _marked = /*#__PURE__*/_regenerator2.default.mark(iterateBundles);

var path = require('path');
var prettifyTime = require('./prettifyTime');
var logger = require('../Logger');
var emoji = require('./emoji');
var filesize = require('filesize');

var LARGE_BUNDLE_SIZE = 1024 * 1024;
var NUM_LARGE_ASSETS = 10;
var COLUMNS = [{ align: 'left' }, // name
{ align: 'right' }, // size
{ align: 'right' // time
}];

function bundleReport(mainBundle) {
  var detailed = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

  // Get a list of bundles sorted by size
  var bundles = (0, _from2.default)(iterateBundles(mainBundle)).sort(function (a, b) {
    return b.totalSize - a.totalSize;
  });
  var rows = [];

  var _loop = function _loop(bundle) {
    // Add a row for the bundle
    rows.push([formatFilename(bundle.name, logger.chalk.cyan.bold), logger.chalk.bold(prettifySize(bundle.totalSize, bundle.totalSize > LARGE_BUNDLE_SIZE)), logger.chalk.green.bold(prettifyTime(bundle.bundleTime))]);

    // If detailed, generate a list of the top 10 largest assets in the bundle
    if (detailed && bundle.assets.size > 1) {
      var assets = (0, _from2.default)(bundle.assets).filter(function (a) {
        return a.type === bundle.type;
      }).sort(function (a, b) {
        return b.bundledSize - a.bundledSize;
      });

      var largestAssets = assets.slice(0, NUM_LARGE_ASSETS);
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = (0, _getIterator3.default)(largestAssets), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var asset = _step2.value;

          // Add a row for the asset.
          rows.push([(asset == assets[assets.length - 1] ? '└── ' : '├── ') + formatFilename(asset.name, logger.chalk.reset), logger.chalk.dim(prettifySize(asset.bundledSize)), logger.chalk.dim(logger.chalk.green(prettifyTime(asset.buildTime)))]);
        }

        // Show how many more assets there are
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      if (assets.length > largestAssets.length) {
        rows.push(['└── ' + logger.chalk.dim('+ ' + (assets.length - largestAssets.length) + ' more assets')]);
      }

      // If this isn't the last bundle, add an empty row before the next one
      if (bundle !== bundles[bundles.length - 1]) {
        rows.push([]);
      }
    }
  };

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = (0, _getIterator3.default)(bundles), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var bundle = _step.value;

      _loop(bundle);
    }

    // Render table
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  logger.log('');
  logger.table(COLUMNS, rows);
}

module.exports = bundleReport;

function iterateBundles(bundle) {
  var _iteratorNormalCompletion3, _didIteratorError3, _iteratorError3, _iterator3, _step3, child;

  return _regenerator2.default.wrap(function iterateBundles$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return bundle;

        case 2:
          _iteratorNormalCompletion3 = true;
          _didIteratorError3 = false;
          _iteratorError3 = undefined;
          _context.prev = 5;
          _iterator3 = (0, _getIterator3.default)(bundle.childBundles);

        case 7:
          if (_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done) {
            _context.next = 13;
            break;
          }

          child = _step3.value;
          return _context.delegateYield(iterateBundles(child), 't0', 10);

        case 10:
          _iteratorNormalCompletion3 = true;
          _context.next = 7;
          break;

        case 13:
          _context.next = 19;
          break;

        case 15:
          _context.prev = 15;
          _context.t1 = _context['catch'](5);
          _didIteratorError3 = true;
          _iteratorError3 = _context.t1;

        case 19:
          _context.prev = 19;
          _context.prev = 20;

          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }

        case 22:
          _context.prev = 22;

          if (!_didIteratorError3) {
            _context.next = 25;
            break;
          }

          throw _iteratorError3;

        case 25:
          return _context.finish(22);

        case 26:
          return _context.finish(19);

        case 27:
        case 'end':
          return _context.stop();
      }
    }
  }, _marked, this, [[5, 15, 19, 27], [20,, 22, 26]]);
}

function prettifySize(size, isLarge) {
  var res = filesize(size);
  if (isLarge) {
    res = logger.chalk.yellow(emoji.warning + '  ' + res);
  } else {
    res = logger.chalk.magenta(res);
  }

  return res;
}

function formatFilename(filename) {
  var color = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : logger.chalk.reset;

  var dir = path.relative(process.cwd(), path.dirname(filename));
  return logger.chalk.dim(dir + (dir ? path.sep : '')) + color(path.basename(filename));
}