'use strict';

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

/**
 * Loads target node and browser versions from the following locations:
 *   - package.json engines field
 *   - package.json browserslist field
 *   - browserslist or .browserslistrc files
 *   - .babelrc or .babelrc.js files with babel-preset-env
 */
var getTargetEngines = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(asset, isTargetApp) {
    var targets, path, compileTarget, pkg, engines, nodeVersion, _browserslist, babelTargets;

    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            targets = {};
            path = isTargetApp ? asset.options.mainFile : asset.name;
            compileTarget = asset.options.target === 'browser' ? 'browsers' : asset.options.target;
            _context.next = 5;
            return asset.getConfig(['package.json'], { path: path });

          case 5:
            pkg = _context.sent;
            engines = pkg && pkg.engines;
            nodeVersion = engines && getMinSemver(engines.node);

            if (!(compileTarget === 'node')) {
              _context.next = 12;
              break;
            }

            // Use package.engines.node by default if we are compiling for node.
            if (typeof nodeVersion === 'string') {
              targets.node = nodeVersion;
            }
            _context.next = 32;
            break;

          case 12:
            if (!(engines && (typeof engines.browsers === 'string' || Array.isArray(engines.browsers)))) {
              _context.next = 16;
              break;
            }

            targets.browsers = engines.browsers;
            _context.next = 31;
            break;

          case 16:
            if (!(pkg && pkg.browserslist)) {
              _context.next = 20;
              break;
            }

            targets.browsers = pkg.browserslist;
            _context.next = 31;
            break;

          case 20:
            _context.next = 22;
            return loadBrowserslist(asset, path);

          case 22:
            _browserslist = _context.sent;

            if (!_browserslist) {
              _context.next = 27;
              break;
            }

            targets.browsers = _browserslist;
            _context.next = 31;
            break;

          case 27:
            _context.next = 29;
            return loadBabelrc(asset, path);

          case 29:
            babelTargets = _context.sent;

            if (babelTargets && babelTargets.browsers) {
              targets.browsers = babelTargets.browsers;
            } else if (babelTargets && babelTargets.node && !nodeVersion) {
              nodeVersion = babelTargets.node;
            }

          case 31:

            // Fall back to package.engines.node for node_modules without any browser target info.
            if (!isTargetApp && !targets.browsers && typeof nodeVersion === 'string') {
              targets.node = nodeVersion;
            }

          case 32:

            // If we didn't find any targets, set some default engines for the target app.
            if (isTargetApp && !targets[compileTarget] && DEFAULT_ENGINES[compileTarget]) {
              targets[compileTarget] = DEFAULT_ENGINES[compileTarget];
            }

            // Parse browser targets
            if (targets.browsers) {
              targets.browsers = browserslist(targets.browsers).sort();
            }

            // Dont compile if we couldn't find any targets

            if (!((0, _keys2.default)(targets).length === 0)) {
              _context.next = 36;
              break;
            }

            return _context.abrupt('return', null);

          case 36:
            return _context.abrupt('return', targets);

          case 37:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function getTargetEngines(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

var loadBrowserslist = function () {
  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(asset, path) {
    var config, browsers;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return asset.getConfig(['browserslist', '.browserslistrc'], {
              path: path,
              load: false
            });

          case 2:
            config = _context2.sent;

            if (!config) {
              _context2.next = 7;
              break;
            }

            browsers = browserslist.readConfig(config);

            if ((typeof browsers === 'undefined' ? 'undefined' : (0, _typeof3.default)(browsers)) === 'object' && !Array.isArray(browsers)) {
              browsers = browsers[process.env.NODE_ENV] || browsers.defaults;
            }

            return _context2.abrupt('return', browsers);

          case 7:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function loadBrowserslist(_x3, _x4) {
    return _ref2.apply(this, arguments);
  };
}();

var loadBabelrc = function () {
  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(asset, path) {
    var config, env;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return asset.getConfig(['.babelrc', '.babelrc.js'], { path: path });

          case 2:
            config = _context3.sent;

            if (!(config && config.presets)) {
              _context3.next = 7;
              break;
            }

            env = config.presets.find(function (plugin) {
              return Array.isArray(plugin) && (plugin[0] === 'env' || plugin[0] === '@babel/env');
            });

            if (!(env && env[1].targets)) {
              _context3.next = 7;
              break;
            }

            return _context3.abrupt('return', env[1].targets);

          case 7:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function loadBabelrc(_x5, _x6) {
    return _ref3.apply(this, arguments);
  };
}();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var browserslist = require('browserslist');
var semver = require('semver');

var DEFAULT_ENGINES = {
  browsers: ['> 1%', 'last 2 versions', 'Firefox ESR'],
  node: '6'
};

function getMinSemver(version) {
  try {
    var range = new semver.Range(version);
    var sorted = range.set.sort(function (a, b) {
      return a[0].semver.compare(b[0].semver);
    });
    return sorted[0][0].semver.version;
  } catch (err) {
    return null;
  }
}

module.exports = getTargetEngines;