'use strict';

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Path = require('path');
var types = require('babel-types');
var matchesPattern = require('./matches-pattern');

var VARS = {
  process: function process(asset) {
    asset.addDependency('process');
    return 'var process = require("process");';
  },
  global: function global() {
    return 'var global = (1,eval)("this");';
  },
  __dirname: function __dirname(asset) {
    return 'var __dirname = ' + (0, _stringify2.default)(Path.dirname(asset.name)) + ';';
  },
  __filename: function __filename(asset) {
    return 'var __filename = ' + (0, _stringify2.default)(asset.name) + ';';
  },
  Buffer: function Buffer(asset) {
    asset.addDependency('buffer');
    return 'var Buffer = require("buffer").Buffer;';
  }
};

module.exports = {
  MemberExpression: function MemberExpression(node, asset) {
    // Inline environment variables accessed on process.env
    if (matchesPattern(node.object, 'process.env')) {
      var key = types.toComputedKey(node);
      if (types.isStringLiteral(key)) {
        var val = types.valueToNode(process.env[key.value]);
        morph(node, val);
        asset.isAstDirty = true;
        asset.cacheData.env[key.value] = process.env[key.value];
      }
    }
  },
  Identifier: function Identifier(node, asset, ancestors) {
    var parent = ancestors[ancestors.length - 2];
    if (VARS.hasOwnProperty(node.name) && !asset.globals.has(node.name) && types.isReferenced(node, parent)) {
      asset.globals.set(node.name, VARS[node.name](asset));
    }
  },
  Declaration: function Declaration(node, asset, ancestors) {
    // If there is a global declaration of one of the variables, remove our declaration
    var identifiers = types.getBindingIdentifiers(node);
    for (var id in identifiers) {
      if (VARS.hasOwnProperty(id) && !inScope(ancestors)) {
        // Don't delete entirely, so we don't add it again when the declaration is referenced
        asset.globals.set(id, '');
      }
    }
  }
};

function inScope(ancestors) {
  for (var i = ancestors.length - 2; i >= 0; i--) {
    if (types.isScope(ancestors[i]) && !types.isProgram(ancestors[i])) {
      return true;
    }
  }

  return false;
}

// replace object properties
function morph(object, newProperties) {
  for (var key in object) {
    delete object[key];
  }

  for (var _key in newProperties) {
    object[_key] = newProperties[_key];
  }
}